#include "masterheader.h"

using namespace std;
const double BACKALPHA = 0.1, LOCALOPTALPHA = 0.05, CLEANUPALPHA = 0.05;
const long BUDGET = 100000;
const int REPS = 5, GAGEN = 3, INITIALNUMREPS = 5, NUMOFCANDIDATES = 5, PRUNINGFREQ = 5, MPA =0, RMDCHOICE = 0;
const bool BACKTRACKING = 0, OCBA = 1, DOMINANTNICHE = 0, CLEANUP = 1, ELITISM = 1;
const double GABUDGETPROP = 0.18;

bool ReadInput(std::ifstream& inputFile, string& output, std::vector<int>& x0, std::vector<double*>& A, std::vector<double>& b, \
				long& budget, int& reps, bool& backtracking, bool& ocba, int& gagen, bool& dominantniche, bool& cleanup, \
				bool& stocsim, double& globaldelta, double& backalpha, double& backdelta, double& localoptalpha, \
				double& localoptdelta, double& cleanupalpha, double& cleanupdelta, int& initialNumReps, \
				int& numOfCandidates, bool& elitism, int& pruning_freq, int& mpamode, int& rmdmode, double& gabudget)
{
	// Constraints are of the form Ax>=b. The first line is dimension and size of constraints
	// The second line is the starting point
	// Then follow constraints per line.
	int dimension, size, backtemp, ocbatemp, dntemp, cleanuptemp, stocsimtemp, elitismtemp, mpatemp, rmdtemp;
	if (!inputFile.is_open())
	{
		cout << "Error: invalid input file" << endl;
		return false;
	}
	string temp;
	inputFile >> temp >> output;
	inputFile >> temp >> budget;
	inputFile >> temp >> reps;
	inputFile >> temp >> backtemp;
	inputFile >> temp >> ocbatemp;
	inputFile >> temp >> gagen;
	inputFile >> temp >> dntemp;
	inputFile >> temp >> cleanuptemp;
	inputFile >> temp >> stocsimtemp;
	inputFile >> temp >> globaldelta;
	inputFile >> temp >> backalpha;
	inputFile >> temp >> backdelta;
	inputFile >> temp >> localoptalpha;
	inputFile >> temp >> localoptdelta;
	inputFile >> temp >> cleanupalpha;
	inputFile >> temp >> cleanupdelta;
	inputFile >> temp >> initialNumReps;
	inputFile >> temp >> numOfCandidates;
	inputFile >> temp >> elitismtemp;
	inputFile >> temp >> pruning_freq;
	inputFile >> temp >> mpatemp;
	inputFile >> temp >> rmdtemp;
	inputFile >> temp >> gabudget;
	require(cleanupdelta >= 0, "Indifference parameter for clean-up must be positive");
	budget = (budget < 0) ? BUDGET : budget;
	reps = (reps < 0) ? REPS : reps;
	backtracking = (backtemp < 0) ? BACKTRACKING : static_cast<bool>(backtemp);
	ocba = (ocbatemp < 0) ? OCBA : static_cast<bool>(ocbatemp);
	gagen = (gagen < 0) ? GAGEN : gagen;
	dominantniche = (dntemp < 0) ? DOMINANTNICHE : static_cast<bool>(dntemp);
	cleanup = (cleanuptemp < 0) ? CLEANUP : static_cast<bool>(cleanuptemp);
	stocsim = (stocsimtemp == 0) ? false : true;
	globaldelta = (globaldelta < 0) ? cleanupdelta : globaldelta;
	backalpha = (backalpha < 0) ? BACKALPHA : backalpha;
	backdelta = (backdelta < 0) ? cleanupdelta : backdelta;
	localoptalpha = (localoptalpha < 0) ? LOCALOPTALPHA : localoptalpha;
	localoptdelta = (localoptdelta < 0) ? cleanupdelta : localoptdelta;
	cleanupalpha = (cleanupalpha < 0) ? CLEANUPALPHA : cleanupalpha;
	//cleanup_delta must be provided, a positive number
	require(cleanupdelta > 0, "Cleanup Delta must be provided and must be a nonnegative number"); 
	initialNumReps = (initialNumReps < 0) ? INITIALNUMREPS : initialNumReps;
	// If it is a stochastic sim problem, require at least 2 initial reps
	if (initialNumReps < 2 && stocsim) initialNumReps = 2;
	numOfCandidates = (numOfCandidates < 0) ? NUMOFCANDIDATES : numOfCandidates;
	elitism = (elitismtemp < 0) ? ELITISM : static_cast<bool>(elitismtemp);
	pruning_freq = (pruning_freq < 0) ? PRUNINGFREQ : pruning_freq;
	mpamode = (mpatemp < 0) ? MPA : mpatemp;
	rmdmode = (rmdtemp < 0) ? RMDCHOICE : rmdtemp;
	gabudget = (gabudget < 0 || gabudget > 1) ? GABUDGETPROP : gabudget;
	inputFile >> temp >> dimension;
	inputFile >> temp >> size;
	for (int i = 0; i < dimension; i++)
	{
		// First read starting point
		int input;
		inputFile >> input;
		x0.push_back(input);
	}
	for (int i = 0; i < size; i++)
	{
		double* Ai = new double[dimension];
		for (int j = 0; j < dimension; j++)
		{
			double input;
			inputFile >> input;
			Ai[j] = input;
		}
		A.push_back(Ai);
		double input;
		inputFile >> input;
		b.push_back(input);
	}
	return true;
}
