#ifndef MPR_H_
#define MPR_H_

#include "masterheader.h"
#include <map>

// visited contains all visited solutions
// starIndex is the index of the current best solution in the vector "visited"
// Ak is the coefficients of the linear constraints, including MPR constraints and original problem formulation constraints
// bk is the RHS of constraints
// A is original problem formulation constraints, only linear constraints are considered
// b is the RHS of the original problem formulation constraints. 
// Constraints are Ak * x >= bk
// Ak, bk, W are the returned values by this function.
// activeSolutions has the indices of those solutions in "visited" that define MPR 

int ConstructMPR(std::vector<Solution*> visited, long starIndex, long oldStarIndex, std::vector<double*>& Ak, \
			std::vector<double>& bk, const std::vector<double*>& A, const std::vector<double>& b, \
			std::vector<long>& activeSolutions, const std::vector<long>& bestSolutions);

void MPRv2(bool stocsim, std::vector<Solution*> visited, long starIndex, long oldStarIndex, std::vector<double*>& Ak, \
			std::vector<double>& bk, const std::vector<double*>& A, const std::vector<double>& b, \
			std::vector<long>& activeSolutions, const std::vector<long>& bestSolutions, double alphac, \
			bool constraintPruning = true);





#endif /*MPR_H_*/
